<?php
/**
 * @package    vBulletin Payment Plugin
 * @author     Hamyar Programming Team
 * @copyright  2010-2014 Mohammad Hossein Abedinpour
 */
class hamyar_payment {
	public $date_obj;
	//no premission
	public function not_allowed() {
		print_no_permission();
	}

	//show error
	public function error($error) {
		if (IS_ADMIN !== true) {
			eval(standard_error($error));
		} else {
			print_cp_message($error, defined('CP_REDIRECT') ? CP_REDIRECT : NULL, 1, defined('CP_BACKURL') ? CP_BACKURL : NULL, defined('CP_CONTINUE') ? true : false);
		}
		exit ;
	}

	//clean inputs
	public function clean($string, $html = false) {
		global $vbulletin;
		$string = trim($string);
		if (!$html) {
			$string = htmlspecialchars($string, ENT_QUOTES);
			$string = strip_tags($string);
		}
		$string = $vbulletin -> db -> escape_string($string);
		return $string;
	}

	//create file
	public function file_write($path, $data, $backup = false) {
		if (file_exists($path) != false) {
			if ($backup) {
				$filenamenew = $path . 'old';
				rename($path, $filenamenew);
			} else {
				unlink($path);
			}
		}
		if ($data != '') {
			$filenum = fopen($path, 'w');
			fwrite($filenum, $data);
			if (fclose($filenum))
				return true;
			return false;
		}
	}

	//get jalali date
	public function get_date($time = null) {
		if (!$time)
			return time();
		if (!is_object($this -> date_obj)) {
			require_once DIR . '/hamyar_payment/jdatetime.class.php';
			$this -> date_obj = new jDateTime(true, true, 'Asia/Tehran');
		}
		return $this -> date_obj -> date("l j F Y H:i", $time);
	}

	//get user info
	public function get_user_info($id, $html = false) {
		if (!$id) {
			if ($html)
				return 'مهمان';
			else
				return array();
		}
		$uid = $this -> clean($id);
		global $vbulletin, $hamyar_payment_ucache;
		static $hamyar_payment_ucache;
		if (!is_array($hamyar_payment_ucache))
			$hamyar_payment_ucache = array();
		if (!isset($hamyar_payment_ucache[$uid])) {
			$hamyar_payment_ucache[$uid] = $vbulletin -> db -> query_first("SELECT * FROM " . TABLE_PREFIX . "user WHERE userid=$uid");
		}
		if (empty($hamyar_payment_ucache[$uid]))
			$hamyar_payment_ucache[$uid] = '-';
		if (!$html)
			return $hamyar_payment_ucache[$uid];
		else
			return "<a href='user.php?do=edit&u={$uid}'>{$hamyar_payment_ucache[$uid]['username']}</a>";
	}

	//language
	public function language($id) {
		$language[16] = 'بعدی';
		$language[17] = 'قبلی';
		$language[18] = 'همه';
		return $language[$id];
	}

	//price error
	public function get_error() {
		if ($this -> currency() == 1)
			return 'مبلغ وارد شده باید عددی و بیشتر از 1000 ریال باشد.';
		return 'مبلغ وارد شده باید عددی و بیشتر از 100 تومان باشد.';
	}

	//final price
	public function final_price($price) {
		if (!is_numeric($price))
			return false;
		if ($this -> currency() == 1)
			return $price / 10;
		return $price;
	}

	//show currency
	public function currency($persian = false) {
		global $vbulletin;
		if (!$persian)
			return $vbulletin -> options['hamyar_payment_currency'];
		elseif ($this -> currency() == 1)
			return 'ریال';
		else
			return 'تومان';
	}

	//get price to show to the user
	public function get_show_price($price) {
		if ($this -> currency() == 1)
			$price = $price * 10;
		$price = vb_number_format($price);
		return $price . ' ' . $this -> currency(true);
	}

	//check price
	public function price_check($price) {
		if ($this -> final_price($price) >= 100)
			return true;
		return false;
	}

	//get form info
	public function get_form($id) {
		$id = (int)$id;
		global $vbulletin;
		return $vbulletin -> db -> query_first("SELECT * FROM " . TABLE_PREFIX . "hamyar_payment_forms WHERE form_id='{$id}' ");
	}

	//get payment info
	public function get_payment($id) {
		$id = (int)$id;
		global $vbulletin;
		return $vbulletin -> db -> query_first("SELECT * FROM " . TABLE_PREFIX . "hamyar_payment_payments WHERE payment_id='{$id}' ");
	}

	//check user is payment form or not
	public function check_payment($formid, $userid) {
		$formid = (int)$formid;
		$userid = (int)$userid;
		global $vbulletin;
		$result = $vbulletin -> db -> query_first("SELECT * FROM " . TABLE_PREFIX . "hamyar_payment_payments WHERE payment_formid='{$formid}' AND payment_userid='{$userid}' AND payment_status=1 ");
		if ($result['payment_id'])
			return true;
		return false;
	}

	//connecting to the gateway
	public function connect($formid, $userid, $date, $userip, $price, $des = NULL, $name = NULL, $email = NULL, $tell = NULL) {
		global $vbulletin;
		$gateway = $vbulletin -> options['hamyar_payment_webservice'];
		$class = 'hamyar_payment_' . $gateway;
		if (!class_exists($class))
			die(json_encode(array(
				'error',
				'Invalid Gateway'
			)));
		if (!class_exists('nusoap_client'))
			require_once DIR . "/hamyar_payment/lib/nusoap.php";
		$gatewayobj = new $class;
		$vbulletin -> db -> query_write("INSERT INTO `" . TABLE_PREFIX . "hamyar_payment_payments`
		(`payment_formid`, `payment_userid`, `payment_status`, `payment_refid`, `payment_orderid`, `payment_gateway`, `payment_date`, `payment_ip`, `payment_price`, `payment_des`, `payment_name`, `payment_email`, `payment_tell`)
		VALUES
		('{$formid}', '{$userid}', 0, NULL, NULL, '{$gateway}', '{$date}', '{$userip}', '{$price}', '{$des}', '{$name}', '{$email}', '{$tell}');");
		$paymentid = $vbulletin -> db -> insert_id();
		$callback = $vbulletin -> options['bburl'] . '/hamyar_payment.php?callback=' . $paymentid;
		$data = $gatewayobj -> connect($userid, $formid, $paymentid, $price, $callback);
		if (defined('SHOW_HTML')) {
			$code = ($data[0] == 'error') ? $data[1] : 'در حال انتقال به درگاه...<script>' . $data[1] . '</script>';
			$this -> error($code);
		} else
			die(json_encode($data));
		exit ;
	}

	//create a string to show for user in correct way :)
	public function nl2br($string) {
		$string = str_replace(array(
			'<br/>',
			'<br />',
			'<br>',
			'</br>'
		), '%hamyar_br%', $string);
		$string = trim($string);
		$string = htmlspecialchars($string, ENT_QUOTES);
		$string = str_replace(array(
			'\r\n',
			'\r',
			'\n'
		), "<br/>", $string);
		$string = stripslashes($string);
		$string = " " . $string;
		@$string = eregi_replace('(((f|ht){1}tp://)[-a-zA-Z0-9@:%_\+.~#?&//=]+)', '<a href="\\1" target=_blank>\\1</a>', $string);
		@$string = eregi_replace('(((f|ht){1}tps://)[-a-zA-Z0-9@:%_\+.~#?&//=]+)', '<a href="\\1" target=_blank>\\1</a>', $string);
		@$string = eregi_replace('([[:space:]()[{}])(www.[-a-zA-Z0-9@:%_\+.~#?&//=]+)', '\\1<a href="http://\\2" target=_blank>\\2</a>', $string);
		@$string = eregi_replace('([_\.0-9a-z-]+@([0-9a-z][0-9a-z-]+\.)+[a-z]{2,3})', '<a href="mailto:\\1" target=_blank>\\1</a>', $string);
		$string = str_replace('%hamyar_br%', '<br/>', $string);
		$string = str_replace(array(
			'<br/>',
			'<br />',
			'<br>',
			'</br>'
		), '<br/>', $string);
		$string = str_replace(array(
			'<br/><br/>',
			'<br/><br/><br/>',
			'<br/><br/><br/><br/>'
		), '<br/>', $string);
		return nl2br(stripslashes($string));
	}

}

//pagination
class hamyar_payment_paginator {
	var $items_per_page;
	var $items_total;
	var $current_page;
	var $num_pages;
	var $mid_range;
	var $low;
	var $high;
	var $limit;
	var $return;
	var $default_ipp = 2;
	var $do;
	function Paginator() {
		$this -> current_page = 1;
		$this -> mid_range = 7;
		$this -> items_per_page = (!empty($_GET['ipp'])) ? (int)$_GET['ipp'] : $this -> default_ipp;
	}

	function paginate($do) {
		global $vbulletin;
		$this -> default_ipp = 10;
		$this -> items_per_page = (!empty($_GET['ipp'])) ? (int)$_GET['ipp'] : $this -> default_ipp;
		$this -> do2 = '?do=' . $do;
		if ($_GET['ipp'] == 'All') {
			$this -> num_pages = ceil($this -> items_total / $this -> default_ipp);
			$this -> items_per_page = $this -> default_ipp;
		} else {
			if (!is_numeric($this -> items_per_page) or $this -> items_per_page <= 0)
				$this -> items_per_page = $this -> default_ipp;
			$this -> num_pages = ceil($this -> items_total / $this -> items_per_page);
		}
		$this -> current_page = (int)$_GET['page'];
		// must be numeric > 0
		if ($this -> current_page < 1 or !is_numeric($this -> current_page))
			$this -> current_page = 1;
		if ($this -> current_page > $this -> num_pages)
			$this -> current_page = $this -> num_pages;
		$prev_page = $this -> current_page - 1;
		$next_page = $this -> current_page + 1;

		if ($this -> num_pages > 10) {
			$this -> return2 = ($this -> current_page != 1 and $this -> items_total >= 10) ? "<a class=\"paginate\" href=\"$_SERVER[PHP_SELF]" . $this -> do2 . "&page=$prev_page&ipp=$this->items_per_page\">« " . hamyar_payment::language(17) . "</a> " : "<span class=\"inactive\" href=\"#\">« " . hamyar_payment::language(17) . "</span> ";

			$this -> start_range = $this -> current_page - floor($this -> mid_range / 2);
			$this -> end_range = $this -> current_page + floor($this -> mid_range / 2);

			if ($this -> start_range <= 0) {
				$this -> end_range += abs($this -> start_range) + 1;
				$this -> start_range = 1;
			}
			if ($this -> end_range > $this -> num_pages) {
				$this -> start_range -= $this -> end_range - $this -> num_pages;
				$this -> end_range = $this -> num_pages;
			}
			$this -> range = range($this -> start_range, $this -> end_range);

			for ($i = 1; $i <= $this -> num_pages; $i++) {
				if ($this -> range[0] > 2 and $i == $this -> range[0])
					$this -> return2 .= " ... ";
				// loop through all pages. if first, last, or in range, display
				if ($i == 1 or $i == $this -> num_pages or in_array($i, $this -> range)) {
					$this -> return2 .= ($i == $this -> current_page and $_GET['page'] != 'All') ? "<a title=\"Go to page $i of $this->num_pages\" class=\"current\" href=\"#\">$i</a> " : "<a class=\"paginate\" title=\"Go to page $i of $this->num_pages\" href=\"$_SERVER[PHP_SELF]" . $this -> do2 . "&page=$i&ipp=$this->items_per_page\">$i</a> ";
				}
				if ($this -> range[$this -> mid_range - 1] < $this -> num_pages - 1 and $i == $this -> range[$this -> mid_range - 1])
					$this -> return2 .= " ... ";
			}
			$this -> return2 .= (($this -> current_page != $this -> num_pages and $this -> items_total >= 10) and ($_GET['page'] != 'All')) ? "<a class=\"paginate\" href=\"$_SERVER[PHP_SELF]" . $this -> do2 . "&page=$next_page&ipp=$this->items_per_page\">" . hamyar_payment::language(16) . " »</a>\n" : "<span class=\"inactive\" href=\"#\">» " . hamyar_payment::language(16) . "</span>\n";
			$this -> return2 .= ($_GET['page'] == 'All') ? "<a class=\"current\" style=\"margin-left:10px\" href=\"#\">" . hamyar_payment::language(15) . "</a> \n" : "<a class=\"paginate\" style=\"margin-left:10px\" href=\"$_SERVER[PHP_SELF]" . $this -> do2 . "&page=1&ipp=All\">" . hamyar_payment::language(18) . "</a> \n";
		} else {
			for ($i = 1; $i <= $this -> num_pages; $i++) {
				$this -> return2 .= ($i == $this -> current_page) ? "<a class=\"current\" href=\"#\">$i</a> " : "<a class=\"paginate\" href=\"$_SERVER[PHP_SELF]" . $this -> do2 . "&page=$i&ipp=$this->items_per_page\">$i</a> ";
			}
			@$this -> return2 .= "<a class=\"paginate\" href=\"$_SERVER[PHP_SELF]" . $this -> do2 . "&page=1&ipp=All\">" . hamyar_payment::language(18) . "</a> \n";
		}
		$this -> low = ($this -> current_page - 1) * $this -> items_per_page;
		$this -> high = ($_GET['ipp'] == 'All') ? $this -> items_total : ($this -> current_page * $this -> items_per_page) - 1;
		$this -> limit = ($_GET['ipp'] == 'All') ? "" : " LIMIT $this->low,$this->items_per_page";
	}

	function display_items_per_page() {
		$items = '';
		$ipp_array = array(
			10,
			25,
			50,
			100,
			'All'
		);
		foreach ($ipp_array as $ipp_opt)
			$items .= ($ipp_opt == $this -> items_per_page) ? "<option selected value=\"$ipp_opt\">$ipp_opt</option>\n" : "<option value=\"$ipp_opt\">$ipp_opt</option>\n";
		return "<select class=\"paginate\" onchange=\"window.location='$_SERVER[PHP_SELF]" . $this -> do2 . "&page=1&ipp='+this[this.selectedIndex].value;return false\">$items</select>\n";
	}

	function display_jump_menu() {
		for ($i = 1; $i <= $this -> num_pages; $i++) {
			$option .= ($i == $this -> current_page) ? "<option value=\"$i\" selected>$i</option>\n" : "<option value=\"$i\">$i</option>\n";
		}
		return "<select class=\"paginate\" onchange=\"window.location='$_SERVER[PHP_SELF]" . $this -> do2 . "&page='+this[this.selectedIndex].value+'&ipp=$this->items_per_page';return false\">$option</select>\n";
	}

	function display_pages() {
		return $this -> return2;
	}

	function set_high() {
		global $vbulletin, $a;
		$this -> high = ($_REQUEST['ipp'] == 'All') ? $this -> items_total : $this -> low + $vbulletin -> db -> num_rows($a);
		if ($this -> high < 0)
			$this -> high = 0;
	}

	function show() {
		$low = $this -> low + 1;
		return "نمایش {$low} تا {$this->high} از {$this->items_total} مورد";
	}

}

//gateways
class hamyar_payment_gateways {
	public function connect($userid, $formid, $paymentid, $price, $callback) {
		return array(
			'error',
			'Debuging'
		);
	}

	public function callback($RefId, $SaleOrderId, $SaleReferenceId, $amount) {
		return array(
			'status' => '0',
			'message' => 'Debuging'
		);
	}

	public function get_refid($res) {
		global $hamyar_payment;
		return $hamyar_payment -> clean($_REQUEST['SaleReferenceId']);
	}

	public function get_codes($orderid, $refid) {
		$return = '<br/>کد رهگیری 1: ' . $orderid;
		if (!empty($refid))
			$return .= '<br/>کد رهگیری 2: ' . $refid;
		return $return;
	}

}

//SibaPal - Dedicate
class hamyar_payment_sibapal2 extends hamyar_payment_gateways {
	public function connect($userid, $formid, $paymentid, $price, $callback) {
		$orderId = $userid . $formid . $paymentid;
		global $vbulletin;
		$params = array(
			'pin' => $vbulletin -> options['hamyar_payment_sibapal'],
			'amount' => $price,
			'callback' => urlencode($callback),
			'order_id' => $orderId,
		);
		$params = json_encode($params);
		if (!function_exists('curl_init')) {
			return array(
				'error',
				'cUrl بر روی سرور نصب نیست.'
			);
		}
		$do = curl_init();
		curl_setopt($do, CURLOPT_URL, "https://sibapal.com/Eserviceapi/request?params={$params}");
		curl_setopt($do, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($do, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($do, CURLOPT_CONNECTTIMEOUT, 20);
		$response = curl_exec($do);
		curl_close($do);
		$res = json_decode($response, true);
		if (!$res) {
			return array(
				'error',
				'اتصال به سیباپال برقرار نشد. ' . curl_error($do)
			);
		} else {
			if ($res['result'] == 1) {
				$au = $res['au'];
				$vbulletin -> db -> query("UPDATE `" . TABLE_PREFIX . "hamyar_payment_payments` SET payment_orderid = '{$orderId}', payment_recid='{$au}' WHERE payment_id = '{$paymentid}'");
				$res['form'] = str_replace(array(
					'"',
					"\n",
					"\r\n",
					"Pay it!"
				), array(
					"'",
					"",
					"",
					"برای پرداخت کلیک کنید."
				), $res['form']);
				if (defined('SHOW_HTML'))
					$code = 'window.parent.document.write("' . $res['form'] . '");siba.submit();';
				else
					$code = 'window.parent.document.write("' . $res['form'] . '<script>siba.submit();</script>")';
				return array(
					'ok',
					$code
				);
			} else {
				return array(
					'error',
					'اتصال به سیباپال برقرار نشد. کد خطا: ' . $res['result']
				);
			}

		}
	}

	public function callback($RefId, $SaleOrderId, $SaleReferenceId, $amount) {
		global $vbulletin;
		$params = array(
			'pin' => $vbulletin -> options['hamyar_payment_sibapal'],
			'amount' => $amount,
			'au' => $RefId,
			'order_id' => $SaleOrderId,
			'bank_return' => $_POST + $_GET,
		);
		$params['bank_return'] = base64_encode(json_encode($params['bank_return']));
		$params = json_encode($params);
		$do = curl_init();
		curl_setopt($do, CURLOPT_URL, "https://sibapal.com/Eserviceapi/verify?params={$params}");
		curl_setopt($do, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($do, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($do, CURLOPT_CONNECTTIMEOUT, 20);
		$response = curl_exec($do);
		curl_close($do);
		$res = json_decode($response, true);
		if (empty($res)) {
			return array(
				'status' => '0',
				'message' => 'اتصال به سیباپال برقرار نشد. کد خطا: ' . curl_error($do),
				'bank_au' => $res['bank_au']
			);
		} elseif ($res['result'] == 1) {
			return array(
				'status' => '1',
				'bank_au' => $res['bank_au']
			);
		} else {
			return array(
				'status' => '0',
				'message' => 'پرداخت انجام نشد. کد خطا: ' . $res['result'] . ' پیغام: ' . urldecode($res['msg']),
				'bank_au' => $res['bank_au']
			);
		}
	}

	public function get_refid($res) {
		global $hamyar_payment;
		return $hamyar_payment -> clean($res['bank_au']);
	}

}

//SibaPal - Direct
class hamyar_payment_sibapal1 extends hamyar_payment_gateways {
	public function connect($userid, $formid, $paymentid, $price, $callback) {
		$orderId = $userid . $formid . $paymentid;
		global $vbulletin;
		$client = new nusoap_client("https://www.sibapal.com/payment/wsdl?wsdl", 'wsdl');
		$res = $client -> call('request', array(
			$vbulletin -> options['hamyar_payment_sibapal'],
			$price,
			$callback,
			$orderId,
			'Hamyar Payment Plugin - Payment Number ' . $paymentid,
			$vbulletin -> options['hamyar_payment_sibapal2'],
			0
		));
		if (strlen($res) > 3) {
			$vbulletin -> db -> query("UPDATE `" . TABLE_PREFIX . "hamyar_payment_payments` SET payment_orderid = '{$orderId}', payment_recid='{$res}' WHERE payment_id = '{$paymentid}'");
			$code = 'window.parent.document.location="https://www.sibapal.com/pay?au=' . $res . '"';
			return array(
				'ok',
				$code
			);
		} else {
			return array(
				'error',
				'اتصال به سیباپال برقرار نشد. کد خطا: ' . $res
			);
		}
	}

	public function callback($RefId, $SaleOrderId, $SaleReferenceId, $amount) {
		$client = new nusoap_client("https://www.sibapal.com/payment/wsdl?wsdl", 'wsdl');
		global $vbulletin;
		$result = $client -> call('verify', array(
			$vbulletin -> options['hamyar_payment_sibapal'],
			$amount,
			$RefId
		));
		$err = array();
		$err[-20] = 'شناسه درگاه نامعتبر است';
		$err[-21] = 'آی پی نامعتبر است';
		$err[-22] = 'مبلغ از کف تعریف شده کمتر است';
		$err[-23] = 'مبلغ از سقف تعریف شده بیشتر است';
		$err[-24] = 'مبلغ نامعتبر است';
		$err[-6] = 'ارتباط با بانک برقرار نشد';
		$err[-26] = 'درگاه غیرفعال است';
		$err[-27] = 'آی پی شما مسدود است';
		$err[-9] = 'خطای ناشناخته';
		$err[-29] = 'آدرس کال بک خالی است ';
		$err[-30] = 'چنین تراکنشی یافت نشد';
		$err[-31] = 'تراکنش انجام نشده ';
		$err[-32] = 'تراکنش انجام شده اما مبلغ نادرست است ';
		$err[-35] = 'شماره order_id نامعتبر است !';
		$err[-36] = 'پارامترهاي ارسالي بانکي نامعتبر هستند';
		$err[-38] = 'شده است verify تراکنش قبلا یکبار';
		if (!empty($result) and $result == 1) {
			return array(
				'status' => '1',
				'message' => ''
			);
		} else {
			return array(
				'status' => '0',
				'message' => "خطا : " . $err[$result]
			);
		}
	}

	public function get_refid($res) {
		global $hamyar_payment;
		return $hamyar_payment -> clean($_GET["au"]);
	}

}


//bitpay developement team
class hamyar_payment_bitpay extends hamyar_payment_gateways {
	public function connect($userid, $formid, $paymentid, $price, $callback) {
		$orderId = $userid . $formid . $paymentid;
		global $vbulletin;
		
		$callback = urlencode($callback);
		$price = $price * 10;
		$api = $vbulletin ->options['hamyar_payment_bitpay'];
		
		$ch = curl_init();
		curl_setopt($ch,CURLOPT_URL,'https://bitpay.ir/payment/gateway-send');
		curl_setopt($ch,CURLOPT_POSTFIELDS,"api=$api&amount=$price&redirect=$callback");
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch,CURLOPT_RETURNTRANSFER,true);
		$res = curl_exec($ch);
		curl_close($ch);
		
		if (is_numeric($res) && $res > 0 ){
			$vbulletin -> db -> query("UPDATE `" . TABLE_PREFIX . "hamyar_payment_payments` SET payment_orderid = '{$orderId}', payment_recid='{$res}' WHERE payment_id = '{$paymentid}'");
			$code = 'window.parent.document.location="https://bitpay.ir/payment/gateway-'.$res.'"';
			return array(
				'ok',
				$code
			);
		} else {
			return array(
				'error',
				'اتصال به بیت پی برقرار نشد. کد خطا: ' . $res
			);
		}
	}

	public function callback($RefId, $SaleOrderId, $SaleReferenceId, $amount) {
		global $vbulletin;
		
		$trans_id = $_POST['trans_id'];
		$id_get = $_POST['id_get'];
		$api = $vbulletin -> options['hamyar_payment_bitpay'];
		
		$ch = curl_init();
		curl_setopt($ch,CURLOPT_URL,'https://bitpay.ir/payment/gateway-result-second');
		curl_setopt($ch,CURLOPT_POSTFIELDS,"api=$api&id_get=$id_get&trans_id=$trans_id");
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch,CURLOPT_RETURNTRANSFER,true);
		$result = curl_exec($ch);
		curl_close($ch);
		
		$vbulletin -> db -> query("UPDATE `" . TABLE_PREFIX . "hamyar_payment_payments` SET payment_refid = '{$trans_id}' WHERE payment_recid = '{$id_get}'");
		if (!empty($result) && $result == 1) {
			
			return array(
				'status' => '1',
				'message' => ''
			);
		} else {
			return array(
				'status' => '0',
				'message' => "خطای بیت پی : $result</br/> شماره تراکنش بیت پی: $trans_id"
			);
		}
	}

	public function get_refid($res) {
		global $hamyar_payment;
		return $hamyar_payment -> clean($_POST['trans_id']);
	}

}



//behpardakht mellat
class hamyar_payment_mellat extends hamyar_payment_gateways {
	var $webservice = 'https://pgws.bpm.bankmellat.ir/pgwchannel/services/pgw?wsdl', $namespace = 'http://interfaces.core.sw.bps.com/';
	public function connect($userid, $formid, $paymentid, $price, $callback) {
		global $vbulletin;
		$terminalId = $vbulletin -> options['hamyar_payment_mellat_id'];
		$userName = $vbulletin -> options['hamyar_payment_mellat_user'];
		$userPassword = $vbulletin -> options['hamyar_payment_mellat_pass'];
		$orderId = $userid . $formid . $paymentid;
		$amount = $price * 10;
		$localDate = date('Ymd');
		$localTime = date('Gis');
		$additionalData = '';
		$callBackUrl = $callback;
		$payerId = 0;
		$parameters = array(
			'terminalId' => $terminalId,
			'userName' => $userName,
			'userPassword' => $userPassword,
			'orderId' => $orderId,
			'amount' => $amount,
			'localDate' => $localDate,
			'localTime' => $localTime,
			'additionalData' => $additionalData,
			'callBackUrl' => $callBackUrl,
			'payerId' => $payerId
		);
		$client = new nusoap_client($this -> webservice);
		$namespace = $this -> namespace;
		$result = $client -> call('bpPayRequest', $parameters, $namespace);
		if ($client -> fault) {
			return array(
				'error',
				'خطا در هنگام اتصال به بانک ملت: ' . $client -> getError()
			);
		} else {
			$err = $client -> getError();
			if ($err) {
				return array(
					'error',
					'خطا در هنگام اتصال به بانک ملت: ' . $err
				);
			} else {
				$res = explode(',', $result);
				$ResCode = $res[0];
				if ($ResCode == "0") {
					$vbulletin -> db -> query("UPDATE `" . TABLE_PREFIX . "hamyar_payment_payments` SET payment_orderid = '{$orderId}', payment_recid='{$res[1]}' WHERE payment_id = '{$paymentid}'");
					$code = 'var form = window.parent.document.createElement("form");form.setAttribute("method", "POST");form.setAttribute("action", "https://bpm.shaparak.ir/pgwchannel/startpay.mellat");form.setAttribute("target", "_self");var hiddenField = document.createElement("input");hiddenField.setAttribute("name", "RefId");hiddenField.setAttribute("value","' . $res[1] . '");form.appendChild(hiddenField);window.parent.document.body.appendChild(form);form.submit();window.parent.document.body.removeChild(form);';
					return array(
						'ok',
						$code
					);
				} else {
					return array(
						'error',
						'خطا در هنگام اتصال به بانک ملت: ' . $result
					);
				}
			}
		}
	}

	public function callback($RefId, $SaleOrderId, $SaleReferenceId, $amount) {
		global $vbulletin;
		if ($_POST['ResCode'] == '0') {
			$client = new nusoap_client($this -> webservice);
			$namespace = $this -> namespace;
			$terminalId = $vbulletin -> options['hamyar_payment_mellat_id'];
			$userName = $vbulletin -> options['hamyar_payment_mellat_user'];
			$userPassword = $vbulletin -> options['hamyar_payment_mellat_pass'];
			$err = $client -> getError();
			if ($err) {
				$output['status'] = 0;
				$output['reversal'] = 1;
				$output['message'] = $err;
			} else {
				$parameters = array(
					'terminalId' => $terminalId,
					'userName' => $userName,
					'userPassword' => $userPassword,
					'orderId' => $SaleOrderId,
					'saleOrderId' => $SaleOrderId,
					'saleReferenceId' => $SaleReferenceId
				);
				$result = $client -> call('bpVerifyRequest', $parameters, $namespace);
				if ($client -> fault) {
					$output['status'] = 0;
					$output['reversal'] = 1;
					foreach ($result as $error)
						$output['message'] .= $error;
				} else {
					$err = $client -> getError();
					if ($err) {
						$output['status'] = 0;
						$output['reversal'] = 1;
						$output['message'] = $err;
					} else {
						if ($result == 0) {
							$err = $client -> getError();
							if ($err) {
								$output['status'] = 0;
								$output['reversal'] = 1;
								$output['message'] = $err;
							} else {
								$parameters = array(
									'terminalId' => $terminalId,
									'userName' => $userName,
									'userPassword' => $userPassword,
									'orderId' => $SaleOrderId,
									'saleOrderId' => $SaleOrderId,
									'saleReferenceId' => $SaleReferenceId
								);
								$result = $client -> call('bpInquiryRequest', $parameters, $namespace);
								if ($result == 0) {
									if ($client -> fault) {
										$output['status'] = 0;
										$output['reversal'] = 1;
										foreach ($result as $error)
											$output['message'] .= $error;
									} else {
										$err = $client -> getError();
										if ($err) {
											$output['status'] = 0;
											$output['reversal'] = 1;
											$output['message'] = $err;
										} else {
											$output['status'] = 1;
										}
									}
								} else {
									$output['status'] = 0;
									$output['reversal'] = 0;
									$output['message'] = $this -> error($result);
								}
							}
						} else {
							$output['status'] = 0;
							$output['reversal'] = 0;
							$output['message'] = $this -> error($result);
						}
					}
				}
			}
		} else {
			$output['status'] = 0;
			$output['reversal'] = 0;
			$output['message'] = 'شما از انجام تراكنش منصرف شدید.';
			$cancel = 1;
		}
		if ($output['status'] == 1) {
			$err = $client -> getError();
			if ($err) {
				$output['status'] = 0;
				$output['message'] = $err;
			} else {
				$parameters = array(
					'terminalId' => $terminalId,
					'userName' => $userName,
					'userPassword' => $userPassword,
					'orderId' => $SaleOrderId,
					'saleOrderId' => $SaleOrderId,
					'saleReferenceId' => $SaleReferenceId
				);
				$result = $client -> call('bpSettleRequest', $parameters, $namespace);

				if ($result == 0) {
					if ($client -> fault) {
						$output['status'] = 0;
						foreach ($result as $error)
							$output['message'] .= $error;
					} else {
						$err = $client -> getError();
						if ($err) {
							$output['reversal'] = 1;
							$output['status'] = 0;
							$output['message'] = $err;
						} else {
							$output['status'] = 1;
						}
					}
				} else {
					$output['status'] = 0;
					$output['reversal'] = 1;
					$output['message'] = $this -> error($result);
				}
			}
		}
		if ($output['reversal'] == 1) {
			$err = $client -> getError();
			if ($err) {
				$output['status'] = 0;
				$output['message'] = $err;
			} else {
				$parameters = array(
					'terminalId' => $terminalId,
					'userName' => $userName,
					'userPassword' => $userPassword,
					'orderId' => $SaleOrderId,
					'saleOrderId' => $SaleOrderId,
					'saleReferenceId' => $SaleReferenceId
				);
				$result = $client -> call('bpReversalRequest', $parameters, $namespace);
				if ($client -> fault) {
					$output['status'] = 0;
					foreach ($result as $error)
						$output['message'] .= $error;
				} else {
					$err = $client -> getError();
					if ($err) {
						$output['status'] = 0;
						$output['message'] = $err;
					} else {
						if ($result == 0) {
							$output['message'] .= '<br />در موقع پرداخت مشکلی به وجود آمد٬ مبلغ پرداخت شده به حساب شما برگشت داده شد.';
						} else {
							$output['message'] .= '<br />در موقع پرداخت مشکلی به وجود آمد.';
						}
					}
				}
			}
		} elseif ($cancel != 1) {
			$output['message'] .= '<br />در موقع پرداخت مشکلی به وجود آمد.';
		}
		if ($output['status'] == 0)
			$output['message'] = '<font color="red">' . $output['message'] . '</font>';
		return $output;
	}

	public function error($ResCode) {
		switch ($ResCode) {
			case '0' :
				$prompt = "&#1578;&#1585;&#1575;&#1705;&#1606;&#1588; &#1576;&#1575; &#1605;&#1608;&#1601;&#1602;&#1610;&#1578; &#1575;&#1606;&#1580;&#1575;&#1605; &#1588;&#1583;.";
				break;
			case '11' :
				$prompt = "&#1588;&#1605;&#1575;&#1585;&#1607; &#1705;&#1575;&#1585;&#1578; &#1606;&#1575;&#1605;&#1593;&#1578;&#1576;&#1585; &#1575;&#1587;&#1578;.";
				break;
			case '12' :
				$prompt = "&#1605;&#1608;&#1580;&#1608;&#1583;&#1610; &#1705;&#1575;&#1601;&#1610; &#1606;&#1610;&#1587;&#1578;.";
				break;
			case '13' :
				$prompt = "&#1585;&#1605;&#1586; &#1606;&#1575;&#1583;&#1585;&#1587;&#1578; &#1575;&#1587;&#1578;.";
				break;
			case '14' :
				$prompt = "&#1578;&#1593;&#1583;&#1575;&#1583; &#1583;&#1601;&#1593;&#1575;&#1578; &#1608;&#1575;&#1585;&#1583; &#1705;&#1585;&#1583;&#1606; &#1585;&#1605;&#1586; &#1662;&#1610;&#1588; &#1575;&#1586; &#1581;&#1583; &#1605;&#1580;&#1575;&#1586; &#1575;&#1587;&#1578;.";
				break;
			case '15' :
				$prompt = "&#1705;&#1575;&#1585;&#1578; &#1606;&#1575;&#1605;&#1593;&#1578;&#1576;&#1585;&#1575;&#1587;&#1578;.";
				break;
			case '17' :
				$prompt = "&#1705;&#1575;&#1585;&#1576;&#1585; &#1575;&#1586; &#1575;&#1606;&#1580;&#1575;&#1605; &#1578;&#1585;&#1575;&#1705;&#1606;&#1588; &#1605;&#1606;&#1589;&#1585;&#1601; &#1588;&#1583;&#1607; &#1575;&#1587;&#1578;.";
				break;
			case '18' :
				$prompt = "&#1578;&#1575;&#1585;&#1610;&#1582; &#1575;&#1606;&#1602;&#1590;&#1575;&#1610; &#1705;&#1575;&#1585;&#1578; &#1711;&#1584;&#1588;&#1578;&#1607; &#1575;&#1587;&#1578;.";
				break;
			case '111' :
				$prompt = "&#1589;&#1575;&#1583;&#1585;&#1705;&#1606;&#1606;&#1583;&#1607; &#1705;&#1575;&#1585;&#1578; &#1606;&#1575;&#1605;&#1593;&#1578;&#1576;&#1585; &#1575;&#1587;&#1578;.";
				break;
			case '112' :
				$prompt = "&#1582;&#1591;&#1575;&#1610; &#1587;&#1608;&#1610;&#1610;&#1670; &#1589;&#1575;&#1583;&#1585;&#1705;&#1606;&#1606;&#1583;&#1607; &#1705;&#1575;&#1585;&#1578;.";
				break;
			case '113' :
				$prompt = "&#1662;&#1575;&#1587;&#1582; &#1575;&#1586; &#1589;&#1575;&#1583;&#1585;&#1705;&#1606;&#1606;&#1583;&#1607; &#1705;&#1575;&#1585;&#1578; &#1583;&#1585;&#1610;&#1575;&#1601;&#1578; &#1606;&#1588;&#1583;.";
				break;
			case '114' :
				$prompt = "&#1583;&#1575;&#1585;&#1606;&#1583;&#1607; &#1705;&#1575;&#1585;&#1578; &#1605;&#1580;&#1575;&#1586; &#1576;&#1607; &#1575;&#1606;&#1580;&#1575;&#1605; &#1575;&#1610;&#1606; &#1578;&#1585;&#1575;&#1705;&#1606;&#1588; &#1606;&#1610;&#1587;&#1578;.";
				break;
			case '21' :
				$prompt = "&#1662;&#1584;&#1610;&#1585;&#1606;&#1583;&#1607; &#1606;&#1575;&#1605;&#1593;&#1578;&#1576;&#1585; &#1575;&#1587;&#1578;.";
				break;
			case '22' :
				$prompt = "&#1578;&#1585;&#1605;&#1610;&#1606;&#1575;&#1604; &#1605;&#1580;&#1608;&#1586; &#1575;&#1585;&#1575;&#1574;&#1607; &#1587;&#1585;&#1608;&#1610;&#1587; &#1583;&#1585;&#1582;&#1608;&#1575;&#1587;&#1578;&#1610; &#1585;&#1575; &#1606;&#1583;&#1575;&#1585;&#1583;.";
				break;
			case '23' :
				$prompt = "&#1582;&#1591;&#1575;&#1610; &#1575;&#1605;&#1606;&#1610;&#1578;&#1610; &#1585;&#1582; &#1583;&#1575;&#1583;&#1607; &#1575;&#1587;&#1578;.";
				break;
			case '24' :
				$prompt = "&#1575;&#1591;&#1604;&#1575;&#1593;&#1575;&#1578; &#1705;&#1575;&#1585;&#1576;&#1585;&#1610; &#1662;&#1584;&#1610;&#1585;&#1606;&#1583;&#1607; &#1606;&#1575;&#1605;&#1593;&#1578;&#1576;&#1585; &#1575;&#1587;&#1578;.";
				break;
			case '25' :
				$prompt = "&#1605;&#1576;&#1604;&#1594; &#1606;&#1575;&#1605;&#1593;&#1578;&#1576;&#1585; &#1575;&#1587;&#1578;.";
				break;
			case '31' :
				$prompt = "&#1662;&#1575;&#1587;&#1582; &#1606;&#1575;&#1605;&#1593;&#1578;&#1576;&#1585; &#1575;&#1587;&#1578;.";
				break;
			case '32' :
				$prompt = "&#1601;&#1585;&#1605;&#1578; &#1575;&#1591;&#1604;&#1575;&#1593;&#1575;&#1578; &#1608;&#1575;&#1585;&#1583; &#1588;&#1583;&#1607; &#1589;&#1581;&#1610;&#1581; &#1606;&#1610;&#1587;&#1578;.";
				break;
			case '33' :
				$prompt = "&#1581;&#1587;&#1575;&#1576; &#1606;&#1575;&#1605;&#1593;&#1578;&#1576;&#1585; &#1575;&#1587;&#1578;.";
				break;
			case '34' :
				$prompt = "&#1582;&#1591;&#1575;&#1610; &#1587;&#1610;&#1587;&#1578;&#1605;&#1610;.";
				break;
			case '35' :
				$prompt = "&#1578;&#1575;&#1585;&#1610;&#1582; &#1606;&#1575;&#1605;&#1593;&#1578;&#1576;&#1585; &#1575;&#1587;&#1578;.";
				break;
			case '41' :
				$prompt = "&#1588;&#1605;&#1575;&#1585;&#1607; &#1583;&#1585;&#1582;&#1608;&#1575;&#1587;&#1578; &#1578;&#1705;&#1585;&#1575;&#1585;&#1610; &#1575;&#1587;&#1578;.";
				break;
			case '42' :
				$prompt = "&#1578;&#1585;&#1575;&#1705;&#1606;&#1588; sale &#1610;&#1575;&#1601;&#1578; &#1606;&#1588;&#1583;.";
				break;
			case '43' :
				$prompt = "&#1602;&#1576;&#1604;&#1575; &#1583;&#1585;&#1582;&#1608;&#1575;&#1587;&#1578; verify &#1583;&#1575;&#1583;&#1607; &#1588;&#1583;&#1607; &#1575;&#1587;&#1578;.";
				break;
			case '44' :
				$prompt = "&#1583;&#1585;&#1582;&#1608;&#1575;&#1587;&#1578; verify  &#1610;&#1575;&#1601;&#1578; &#1606;&#1588;&#1583;.";
				break;
			case '45' :
				$prompt = "&#1578;&#1585;&#1575;&#1705;&#1606;&#1588; settle &#1588;&#1583;&#1607; &#1575;&#1587;&#1578;.";
				break;
			case '46' :
				$prompt = "&#1578;&#1585;&#1575;&#1705;&#1606;&#1588; settle &#1606;&#1588;&#1583;&#1607; &#1575;&#1587;&#1578;.";
				break;
			case '47' :
				$prompt = "&#1578;&#1585;&#1575;&#1705;&#1606;&#1588; settle &#1610;&#1575;&#1601;&#1578; &#1606;&#1588;&#1583;.";
				break;
			case '48' :
				$prompt = "&#1578;&#1585;&#1575;&#1705;&#1606;&#1588; reverse &#1588;&#1583;&#1607; &#1575;&#1587;&#1578;.";
				break;
			case '49' :
				$prompt = "&#1578;&#1585;&#1575;&#1705;&#1606;&#1588; refund &#1610;&#1575;&#1601;&#1578; &#1606;&#1588;&#1583;.";
				break;
			case '412' :
				$prompt = "&#1588;&#1606;&#1575;&#1587;&#1607; &#1602;&#1576;&#1590; &#1606;&#1575;&#1583;&#1585;&#1587;&#1578; &#1575;&#1587;&#1578;.";
				break;
			case '413' :
				$prompt = "&#1588;&#1606;&#1575;&#1587;&#1607; &#1662;&#1585;&#1583;&#1575;&#1582;&#1578; &#1606;&#1575;&#1583;&#1585;&#1587;&#1578; &#1575;&#1587;&#1578;.";
				break;
			case '414' :
				$prompt = "&#1587;&#1575;&#1586;&#1605;&#1575;&#1606; &#1589;&#1575;&#1583;&#1585;&#1705;&#1606;&#1606;&#1583;&#1607; &#1602;&#1576;&#1590; &#1606;&#1575;&#1605;&#1593;&#1578;&#1576;&#1585; &#1575;&#1587;&#1578;.";
				break;
			case '415' :
				$prompt = "&#1586;&#1605;&#1575;&#1606; &#1580;&#1604;&#1587;&#1607; &#1705;&#1575;&#1585;&#1610; &#1576;&#1607; &#1662;&#1575;&#1610;&#1575;&#1606; &#1585;&#1587;&#1610;&#1583;&#1607; &#1575;&#1587;&#1578;.";
				break;
			case '416' :
				$prompt = "&#1582;&#1591;&#1575; &#1583;&#1585; &#1579;&#1576;&#1578; &#1575;&#1591;&#1604;&#1575;&#1593;&#1575;&#1578;.";
				break;
			case '417' :
				$prompt = "&#1588;&#1606;&#1575;&#1587;&#1607; &#1662;&#1585;&#1583;&#1575;&#1582;&#1578; &#1705;&#1606;&#1606;&#1583;&#1607; &#1606;&#1575;&#1605;&#1593;&#1578;&#1576;&#1585;&#1575;&#1587;&#1578;.";
				break;
			case '418' :
				$prompt = "&#1575;&#1588;&#1705;&#1575;&#1604; &#1583;&#1585; &#1578;&#1593;&#1585;&#1610;&#1601; &#1575;&#1591;&#1604;&#1575;&#1593;&#1575;&#1578; &#1605;&#1588;&#1578;&#1585;&#1610;.";
				break;
			case '419' :
				$prompt = "&#1578;&#1593;&#1583;&#1575;&#1583; &#1583;&#1601;&#1593;&#1575;&#1578; &#1608;&#1585;&#1608;&#1583; &#1575;&#1591;&#1604;&#1575;&#1593;&#1575;&#1578; &#1575;&#1586; &#1581;&#1583; &#1605;&#1580;&#1575;&#1586; &#1711;&#1584;&#1588;&#1578;&#1607; &#1575;&#1587;&#1578;.";
				break;
			case '421' :
				$prompt = "IP &#1606;&#1575;&#1605;&#1593;&#1578;&#1576;&#1585; &#1575;&#1587;&#1578;.";
				break;
			case '51' :
				$prompt = "&#1578;&#1585;&#1575;&#1705;&#1606;&#1588; &#1578;&#1705;&#1585;&#1575;&#1585;&#1610; &#1575;&#1587;&#1578;.";
				break;
			case '52' :
				$prompt = "&#1587;&#1585;&#1608;&#1610;&#1587; &#1583;&#1585;&#1582;&#1608;&#1575;&#1587;&#1578;&#1610; &#1605;&#1608;&#1580;&#1608;&#1583; &#1606;&#1605;&#1610; &#1576;&#1575;&#1588;&#1583;.";
				break;
			case '54' :
				$prompt = "&#1578;&#1585;&#1575;&#1705;&#1606;&#1588; &#1605;&#1585;&#1580;&#1593; &#1605;&#1608;&#1580;&#1608;&#1583; &#1606;&#1610;&#1587;&#1578;.";
				break;
			case '55' :
				$prompt = "&#1578;&#1585;&#1575;&#1705;&#1606;&#1588; &#1606;&#1575;&#1605;&#1593;&#1578;&#1576;&#1585; &#1575;&#1587;&#1578;.";
				break;
			case '61' :
				$prompt = "&#1582;&#1591;&#1575; &#1583;&#1585; &#1608;&#1575;&#1585;&#1610;&#1586;.";
				break;
			default :
				$prompt = "&#1582;&#1591;&#1575;&#1610; &#1606;&#1575;&#1605;&#1588;&#1582;&#1589;.";
				break;
		}
		return '&#1705;&#1583; ' . $ResCode . ' : ' . $prompt;
	}

}
